#
# (C) Tenable Network Security
#
#

if ( ! defined_func("bn_random") ) exit(0);
if(description)
{
 script_id(12634);
 script_version ("$Revision: 1.1 $");
 name["english"] = "Use SSH to perform local security checks";
 
 script_name(english:name["english"]);
 
 desc["english"] = "
This scripts logs into the remote host using SSH and extracts the list
of installed packages. To work properly, this script requires
that you configure it with a valid SSH public key to use and
eventually an SSH passphrase if the SSH public key is passphrase-protected.";


 script_description(english:desc["english"]);
 
 summary["english"] = "Logs into the remote host using SSH";
 script_summary(english:summary["english"]);
 
 script_category(ACT_GATHER_INFO);
 
 script_copyright(english:"This script is Copyright (C) 2004 Tenable Network Security");
 family["english"] = "General";
 script_family(english:family["english"]);
 
 script_dependencies("find_service.nes");
 script_require_ports(22, "Services/ssh");
if (  defined_func("bn_random") ) {
 script_add_preference(name:"SSH user name : ",
		       type:"entry",
		       value:"root");
 script_add_preference(name:"SSH public key to use : ",
		       type:"file",
		       value:"");
 script_add_preference(name:"SSH private key to use : ",
		       type:"file",
		       value:"");
 script_add_preference(name:"Passphrase for SSH key : ",
		       type:"password",
		       value:"");
 }
 exit(0);
}



include("ssh_func.inc");
include("global_settings.inc");

account     = script_get_preference("SSH user name : ");
public_key  = script_get_preference_file_content("SSH public key to use : ");
private_key = script_get_preference_file_content("SSH private key to use : ");
passphrase  = script_get_preference("Passphrase for SSH key : ");

if ( ! account || ! public_key || ! private_key ) exit(0);

public_key = ereg_replace(pattern:"[^ ]* ([^ ]*) [^ ]*$",
			  string:public_key,
			  replace:"\1");




port = kb_ssh_transport();
sock = open_sock_tcp(port);
if (!sock) exit(0);

report = "";

ret = ssh_login(socket:sock, login:account, password:NULL, pub:public_key, priv:private_key, passphrase:passphrase);



if (ret != 0) {
	display("ret =", ret, "\n");
	if ( report_verbosity > 1 )
	{
	 report = string("It was not possible to log into the remote host using the provided SSH public/private key pair (ret=", ret, ")");
	 security_note(port:port, data:report);
	}
	exit(0);
	}


report = "It was possible to log into the remote host using the supplied public key";

# Determine the remote operating system type
buf = ssh_cmd(socket:sock,cmd:"uname -a");
if ( buf ) set_kb_item(name:"Host/uname", value:buf);
else {
	report += 
'\nHowever the execution of the command "uname -a" failed, so local security
checks have not been enabled';
	security_note(port:port, data:report);
	exit(0);
     }


report += '\nThe output of "uname -a" is :\n' + buf;


############################# FreeBSD ###########################################
if ( "FreeBSD" >< buf )
 {
  release = ereg_replace(pattern:".*FreeBSD ([0-9]\.[^ ]*).*", replace:"\1", string:buf);
 items = split(release, sep:"-", keep:0);
 if ( "p" >< items[2] ) items[2] = ereg_replace(pattern:"p", replace:"_", string:items[2]);
 release = "FreeBSD-" + items[0] + items[2];
 set_kb_item(name:"Host/FreeBSD/release", value:release); 
  buf = ssh_cmd(socket:sock, cmd:"pkg_info", timeout:60);
  if ( ! buf )  {
	report += 
'\nThe command "pkg_info" did not return any result, therefore
FreeBSD local security checks have not been enabled for
this test';
	security_note(port:port, data:report);
	}
  else {
        set_kb_item(name:"Host/FreeBSD/pkg_info", value:buf);
        report += '\nLocal security checks have been enabled for this host.';
	security_note(port:port, data:report);
	exit(0);
	}
 }
######################## RedHat Linux ###########################################
else if ("Linux" >< buf )
 {
  buf = ssh_cmd(socket:sock, cmd:"cat /etc/redhat-release");
  if ( egrep(pattern:"Red Hat.*(Enterprise|Advanced).*release (3|2\.1)", string:buf) )
  {
   report += '\nThe remote Red Hat system is :\n' + buf;
   set_kb_item(name:"Host/RedHat/release", value:buf);
   buf = ssh_cmd(socket:sock, cmd:"rpm -qa --qf '%{NAME}-%{VERSION}-%{RELEASE}|%{EPOCH}\n'", timeout:60);

   if ( ! buf )
   {
     report += 
'\nThe command "rpm -qa" did not produce any result, therefore local security 
checks have been disabled';
    security_note(port:port, data:report);
    exit(0);
   }

   report += '\nLocal security checks have been enabled for this test.';
   set_kb_item(name:"Host/RedHat/rpm-list", value:buf);
   security_note(port:port, data:report);
   exit(0);
  }
  else {
	report += 
'\nThe remote Linux host is not a Red Hat Enterprise 2.1 or 3, therefore local 
security checks have not been enabled';
	security_note(port:port, data:report);
	exit(0);
	}
 }
######################## MacOS X ###########################################
else if ("Darwin" >< buf )
 {
  operating_system = ereg_replace(pattern:"^.*Darwin Kernel Version ([0-9]+\.[0-9]+\.[0-9]+):.*$", string:buf, replace:"\1");

  num = split(operating_system, sep:".", keep:FALSE);
  version = "Mac OS X 10." + string(int(num[0]) - 4) + "." + int(num[1]);


  buf = ssh_cmd(socket:sock, cmd:"ls /private/etc/sysctl-macosxserver.conf");
  if ( strlen(buf) ) version = version + " Server";
  set_kb_item(name:"Host/MacOSX/Version", value:version);
  buf = ssh_cmd(socket:sock, cmd:"ls /Library/Receipts");
  if ( ! buf )
  {
   report += 
'\nIt was not possible to get the list of installed package on the 
remote MacOS X system, therefore local security checks have
been disabled';
   security_note(port:port, data:report);
   exit(0);
  }
  set_kb_item(name:"Host/MacOSX/packages", value:buf);
  report += '\nLocal security checks have been enabled for this test.';
  security_note(port:port, data:report);
 }
######################## Solaris 8 ###########################################
else if ( egrep(pattern:"SunOS.*5\.8.*sparc", string:buf) )
{
 set_kb_item(name:"Host/Solaris/Version", value:"Solaris 8 SPARC");
 buf = ssh_cmd(socket:soc, cmd:"showrev -a");
 if ( ! buf )
 {
  report += 
'\nIt was not possible to gather the list of installed packages on the
remote Solaris 8 system, therefore local security checks have been disabled.';
  security_note(port:port, data:report);
  exit(0);
 }

 set_kb_item(name:"Host/Solaris/showrev", string:buf);
 buf = ssh_cmd(socket:soc, cmd:"pkginfo");
 if ( buf ) set_kb_item(name:"Host/Solaris/pkginfo", value:buf);
  report += '\nLocal security checks have been enabled for this test.';
  security_note(port:port, data:report);
}

close(sock);
