#
# This script was written by George A. Theall, <theall@tifaware.com>.
#
# See the Nessus Scripts License for details.
#

if(description)
{
  script_id(12263);
  script_version ("$Revision: 1.1 $");
 
  name["english"] = "IMP Content-Type XSS Vulnerability";
  script_name(english:name["english"]);
 
  desc["english"] = "
The remote server is running an IMP version between 2.0 and 3.2.3
inclusive.  These versions are vulnerable to a cross-scripting attack
whereby an attacker can cause a victim to unknowingly run arbitrary
Javascript code simply by reading a MIME message with a specially
crafted Content-Type header.

Note: Nessus has determined the vulnerability exists on the target
simply by looking at the version number of IMP installed there; it 
has not attempted to actually exploit the vulnerability.

Solution: Upgrade to IMP version 3.2.4 or later.

Risk factor : Serious";
  script_description(english:desc["english"]);
 
  summary["english"] = "Checks for Content-Type XSS vulnerability in IMP";
  script_summary(english:summary["english"]);
 
  script_category(ACT_GATHER_INFO);
  script_copyright(english:"This script is Copyright (C) 2004 George A. Theall");

  family["english"] = "CGI abuses";
  script_family(english:family["english"]);

  script_dependencie("http_version.nasl", "cross_site_scripting.nasl");
  script_require_ports("Services/www", 80);

  exit(0);
}

include("http_func.inc");
include("http_keepalive.inc");

debug = 0;

port = get_http_port(default:80);
if (!get_port_state(port)) exit(0);

if(get_kb_item(string("www/", port, "/generic_xss"))) exit(0);
if (!can_host_php(port:port)) exit(0);

# Search for page in a couple of different locations in addition to cgi_dirs().
# nb: "webmail" is not a standard but is commonly used.
dirs = make_list("/imp", "/horde/imp", "/webmail", cgi_dirs());
foreach d (dirs) {
  # Search in a couple of different pages.
  files = make_list(
    "/services/help/?module=imp&show=about", 
    "/README", "/test.php", "/lib/version.phps", 
    "/status.php3"
  );
  foreach f (files) {
    if (debug) display("debug: testing ", d, f, "...\n");

    # Get the page.
    req = http_get(item:string(d, f), port:port);
    res = http_keepalive_send_recv(port:port, data:req);
    if (res == NULL) exit(0);           # can't connect
    if (debug) display("debug: res =>>", res, "<<\n");
    if (egrep(pattern:"404 Not Found", string:res, icase:TRUE)) break;

    # Specify pattern used to identify version string.
    # - version 4.x
    if (f =~ "^/services/help") {
      pat = ">This is Imp (.+)\.<";
    }
    # - version 3.x
    #   nb: test.php available is itself a vulnerability but sometimes available.
    else if (f == "/test.php") {
      pat = "^ *<li>IMP: +(.+) *</li> *$";
    }
    #   nb: README is not guaranteed to be available.
    else if (f = "/README") {
      pat = "^Version +(.+) *$";
    }
    #   nb: another security risk -- ability to view PHP source.
    else if (f = "/lib/version.phps") {
      pat = "IMP_VERSION', '(.+)'";
    }
    # - version 2.x
    else if (f = "/status.php3") {
      pat = ">IMP, Version (.+)<";
    }
    # - someone updated files but forgot to add a pattern???
    else {
      display("Don't know how to handle file '", f, "'!\n");
      exit(1);
    }

    # Get the version string.
    if (debug) display("debug: grepping results for =>>", pat, "<<\n");
    matches = egrep(pattern:pat, string:res);
    if (!matches) break;
    foreach match (split(matches)) {
      match = chomp(match);
      if (debug) display("debug: grepping >>", match, "<< for =>>", pat, "<<\n");
      vers = eregmatch(pattern:pat, string:match);
      if (vers == NULL) break;
      if (debug) display("debug: IMP version =>>", vers[1], "<<\n");

      if (ereg(pattern:"^(2\.|3\.(0|1|2|2\.[1-3]))$", string:vers[1])) security_hole(port);

      # No sense testing any further since we got the version number.
      exit(0);
    }
  }
}
