#
# This script was written by George A. Theall, <theall@tifaware.com>.
#
# See the Nessus Scripts License for details.
#

if(description)
{
  script_id(12262);
  script_version ("$Revision: 1.2 $");
 
  name["english"] = "Open WebMail Content-Type XSS Vulnerability";
  script_name(english:name["english"]);
 
  desc["english"] = "
The remote server is running Open WebMail version 2.32 or earlier which
is vulnerable to a cross site scripting attack whereby an attacker can
cause a victim to unknowingly run arbitrary Javascript code by reading a
MIME message with a specially crafted Content-Type or
Content-Description header. 

For a desciption of a similar vulnerability as it applies to
SquirrelMail (with exploitation details), see:

  - http://www.rs-labs.com/adv/RS-Labs-Advisory-2004-1.txt

Note: Nessus has determined the vulnerability exists on the target
simply by looking at the version number of Open WebMail installed there;
it has not attempted to actually exploit the vulnerability. 

Solution: Upgrade to Open WebMail version 2.33 or later.
Risk factor : Serious";
  script_description(english:desc["english"]);
 
  summary["english"] = "Checks for Content-Type XSS vulnerability in Open WebMail";
  script_summary(english:summary["english"]);
 
  script_category(ACT_GATHER_INFO);
  script_copyright(english:"This script is Copyright (C) 2004 George A. Theall");

  family["english"] = "CGI abuses";
  script_family(english:family["english"]);

  script_dependencie("http_version.nasl", "cross_site_scripting.nasl");
  script_require_ports("Services/www", 80);

  exit(0);
}

include("http_func.inc");
include("http_keepalive.inc");

debug = 0;

port = get_http_port(default:80);
if (!get_port_state(port)) exit(0);

if(get_kb_item(string("www/", port, "/generic_xss"))) exit(0);

# Search for page in a couple of different locations.
dirs = make_list("", "/openwebmail", cgi_dirs());
foreach d (dirs) {
  # Search in a couple of different pages.
  files = make_list("/", "/openwebmail.pl", "/doc/changes.txt");
  foreach f (files) {
    if (debug) display("debug: testing ", d, f, "...\n");

    # Get the page.
    req = http_get(item:string(d, f), port:port);
    res = http_keepalive_send_recv(port:port, data:req);
    if (res == NULL) exit(0);           # can't connect
    if (debug) display("debug: res =>>", res, "<<\n");
    if (egrep(pattern:"404 Not Found", string:res, icase:TRUE)) break;
    # nb: make sure it uses the Open WebMail title.
    if (egrep(pattern:"TITLE>.*Open WebMail", string:res, icase:TRUE)) break;

    # Specify pattern used to identify version string.
    if (f =~ "/" || f =~ "openwebmail\.pl$") {
      pat = "version (.+)$";
    }
    else if (f =~ "/changes\.txt$") {
      # "mm/dd/yyyy (version x.yy)"
      pat = "([0-1][1-9]/[0-3][0-9]/20[0-9][0-9] .version (.+)$";
    }
    # - someone updated files but forgot to add a pattern???
    else {
      display("Don't know how to handle file '", f, "'!\n");
      exit(1);
    }

    # Get the version string.
    if (debug) display("debug: grepping results for =>>", pat, "<<\n");
    matches = egrep(pattern:pat, string:res);
    if (!matches) break;
    foreach match (split(matches)) {
      match = chomp(match);
      if (debug) display("debug: grepping >>", match, "<< for =>>", pat, "<<\n");
      vers = eregmatch(pattern:pat, string:match);
      if (vers == NULL) break;
      if (debug) display("debug: version =>>", vers[1], "<<\n");

      if (ereg(pattern:"^(1|2\.(0|1|2|3[0-2]))$", string:vers[1])) security_hole(port);

      # No sense testing any further since we got the version number.
      exit(0);
    }
  }
}
